/* (C) Marcin Kwadrans <quar@vitea.pl> */

#include <string>
#include "include/support.h"
#include "include/program.h"
#include "include/environment.h"
#include "include/paritycommand.h"

typedef GNode *GNodePtr;

static GtkWidget *copy_piece_widget (LWPiece *piece)
{
	LWPiece *copied = new LWPiece (piece, piece->row);
	return copied->getWidget();
}

static GtkWidget *widget_tree (GNode *node)
{
	GtkWidget *vbox = gtk_vbox_new (FALSE, 0);

	if (node == NULL) return vbox;
	
	if (node->data != NULL)
		gtk_box_pack_start (GTK_BOX (vbox), copy_piece_widget ((LWPiece *) node->data), FALSE, FALSE, 0);
	
	GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
	
	for (GNode *n = node->children; n != NULL; n=n->next)
		gtk_box_pack_start (GTK_BOX(hbox), widget_tree (n), TRUE, TRUE, 0);

	return vbox;
}
/* False - kontynuje pętle */
static void compare_parity (gboolean **r, LWCommand *cmd_a, LWCommand *cmd_b)
{
	/* cmd_a = ")" , cmd_b = "("*/
	LWParityCommand *pcmd_a=NULL;
	LWParityCommand *pcmd_b=NULL;
	
	if (cmd_a != NULL)
		if (TRUE == cmd_a->isParity())
			pcmd_a = dynamic_cast <LWParityCommand *> (cmd_a);

	if (cmd_b != NULL)
		if (TRUE == cmd_b->isParity())
			pcmd_b = dynamic_cast <LWParityCommand *> (cmd_b);

	if ((pcmd_a != NULL) && (pcmd_b != NULL))
		if (FALSE == pcmd_b->isAlreadyMatched())
			if ((FALSE == pcmd_a->isOpen()) && (TRUE == pcmd_b->isOpen())) {
				pcmd_b->matchWith(cmd_a);
				**r = TRUE;
				return;
			}
			
	if (pcmd_b != NULL) 
		if (FALSE == pcmd_b->isAlreadyMatched())
			if (TRUE == pcmd_b->isOpen ()) {
				**r = TRUE;
				return;
			}
				
	if (pcmd_a != NULL)
		if (FALSE == pcmd_a->isOpen ()) {
			**r = FALSE;
			return;
		}
			
	*r = NULL;
}

static gboolean compare_priority (LWPiece *a, LWPiece *b) 
{

//	if (b == NULL) return TRUE;
	
	LWCommand *cmd_a = a->getCommand();
	LWCommand *cmd_b = b->getCommand();

	gboolean bo;
	gboolean *r = &bo;
	
	compare_parity (&r, cmd_a, cmd_b);

	if (r != NULL) 
		return bo;
	
	guint prio_a = (cmd_a != NULL) ? cmd_a->getPriority() : 16;
	guint prio_b = (cmd_b != NULL) ? cmd_b->getPriority() : 16;
	
	LWLink link = (cmd_a != NULL) ? cmd_a->getLinkType() : LW_LINK_RIGHT;
	
	switch (link) {
		case LW_LINK_LEFT:
			return (prio_a > prio_b) ? TRUE : FALSE;
		case LW_LINK_RIGHT:
			return (prio_a >= prio_b) ? TRUE : FALSE;
	}

	g_return_val_if_reached (FALSE);
}

/* \brief Zmiana polecenia skojarzonego z klockiem 

	Zmienia polecenie skojarzone z klockiem

	\param piece Klocek z poleceniem do zmiany
	\param cmd Nazwa polecenia, które ma być skojarzone z klockiem 
*/
static void toggle_to_cmd (LWPiece *piece, gchar *cmd) 
{
	if (strcmp(piece->getCommand ()->getName(), cmd)) {
		piece->setCommand (cmd);
	}
}

static void lex_scan (LWPiece *prev, LWPiece *piece)
{
	if (prev == NULL) return;
	g_return_if_fail (piece != NULL);
	
	LWCommand *pcmd = prev->getCommand();
	LWCommand *cmd = piece->getCommand();
	
	if (cmd == NULL)
		return;
	
	if (!strcmp (cmd->getName(), "opp") || !strcmp (cmd->getName(), "sub")) {

		if (pcmd == NULL) {
			toggle_to_cmd (piece, "sub");
			return;
		}
		
		if (!strcmp(pcmd->getName(), ")")) {
			toggle_to_cmd (piece, "sub");
			return;
		}
	}
	
	if (!strcmp (cmd->getName(), "sub")) {
		toggle_to_cmd (piece, "opp");
	}
}

/* \brief Pojedyncza iteraca tworzenia drzewa składniowego.
   
   Funkcja wstawia klocek piece do drzewa składniowego,
   znajdując odpowiednie miejsce na podstawia poprzednio
   wstawianego elementu prev. Funkcja realizuje produkcję
   drzewa przy użyciu tzw. metody operatorów.

   \param prev Poprzednio wstawiany element
   \param piece Wstawiany kolecek który może zawierajać polecenie
*/
static GNode *parse_iteration (GNode *prev, LWPiece *piece)
{
GNode *node;

	if (TRUE == compare_priority (piece, (LWPiece *) prev->data)) 
		return g_node_append_data (prev, (gpointer) piece);

	do {
	 	prev = prev->parent;
	} while (FALSE == compare_priority (piece, (LWPiece *) prev->data));

	if (TRUE == piece->getCommand()->isParity())
		if (FALSE == dynamic_cast <LWParityCommand *> (piece->getCommand())->isOpen())
			return prev;	
		/* Wstawienie zamykającej części operatora parzystego np. nawiasu */
		//			return g_node_append_data (prev->parent, (gpointer) piece);

		
	/* Polecenia o niskim priorytecie, ustaw jedno za drugim */
	if (piece->getCommand()->getPriority() <= 1)
		return g_node_append_data (prev, (gpointer) piece);
		
	/* Umieszczanie elementu piece pomiędzy prev, a ostatnim dzieckiem prev */
	node = g_node_last_child (prev); 
	g_node_unlink (node);
	GNode *t = g_node_append_data (prev, (gpointer) piece);
	g_node_append (t, node);
	return t;
}

static gboolean unset_command (GNode *node)
{
	LWPiece *piece = (LWPiece *) node->data;
	
	if (piece == NULL) 
		return FALSE;
	
	LWCommand *cmd = piece->getCommand();
	
	if (cmd != NULL)
		cmd->reset();
	
	return FALSE;
}

static gboolean delete_cb (LWProgram *program)
{
	if (TRUE == program->isFinished())
		delete program;
	else
		program->stop ();
	
	return TRUE;
}

/*! \brief Konstruktor

	Tworzy program
*/
LWProgram::LWProgram () 
: world(NULL), finished(FALSE), stoped(FALSE), tree_piece(NULL), begin_piece(NULL)
{
	context.wizard = NULL;
	context.stack = g_queue_new ();

	for (guint i=0; i <= (guint) 'z' - 'a'; i++) 
		vars[i] = new LWVariable(i);
	
	LWEnvironment::registerProgram (this);
}

/*! \brief Destruktor
	Niszczy zbiór ikon
*/	
LWProgram::~LWProgram ()
{
GtkWidget *window=NULL;

	if (begin_piece != NULL)
		delete begin_piece->row;
	
	if (world != NULL)
		window = gtk_widget_get_ancestor(world->getWidget(), GTK_TYPE_WINDOW);

	if (context.wizard != NULL)
		delete context.wizard;
	
	if (world != NULL)
		delete world;
	
	for (guint i=0; i <= (guint) 'z' - 'a'; i++)
		delete vars[i];
	
	if (window != NULL)
		gtk_widget_destroy (window);
	
	LWEnvironment::unregisterProgram ();
}

/*! \brief Przetwarzanie wartości

	Ze zbioru klocków reprezuntujących cyfry, zmienne, obrazki
	tworzy obiekt wartości, który może być przekazany do polecenia
	\param node Węzeł drzewa z klockami
	\return Stworzona wartość
*/
LWValue *LWProgram::computeValue (GNode *node)
{
	LWValue *value;
	
	LWPiece *piece = (LWPiece *) node->data;
	g_return_val_if_fail (piece != NULL, NULL);

	LWPixmap *pixmap = piece->getBackgroundPixmap();
	g_return_val_if_fail (pixmap != NULL, NULL);

	gboolean b = LWVariable::isPixmapVariable (pixmap);
	
	if (b == TRUE) {
		gint num = LWVariable::getPixmapNum(pixmap);
		g_return_val_if_fail (num >= 0, NULL);
		
		if (node->children == NULL)
			return vars[num];
		else {
			LWMessage *m = new LWMessage (LW_ERROR_BadString);
			m->setPiece (piece);
			throw m;
		}
		
	} else {
		value = new LWValue (pixmap);
	}		
		
	for (GNode *n=node->children; n != NULL; n = n->children) {
		piece = (LWPiece *) n->data;
		g_return_val_if_fail (piece != NULL, NULL);

		pixmap = piece->getBackgroundPixmap();
		g_return_val_if_fail (pixmap != NULL, NULL);
				
		b = LWVariable::isPixmapVariable (pixmap);
					
		if (b == FALSE)
			value->append (pixmap);
		else {
			delete value;
			LWMessage *m = new LWMessage (LW_ERROR_BadString);
			m->setPiece (piece);
			throw m;
		}			
	}
	
	return value;		
}

/*! \brief Ustalenie argument dla polecenia

	Ustala argument dla polecenia
	\param cmd Polecenie dla którego będzie ustalony parametr
	\param node Węzeł, na podstawie którego zostanie obliczona wartość parametru
*/
void LWProgram::setArguments (LWCommand *cmd, GNode *node)
{
gint args=0;
GSList *destroy_list=NULL;
	
	for (GNode *n = node; n != NULL; n = n->next) {
		LWPiece *piece = (LWPiece *) n->data;
		g_return_if_fail (piece != NULL);
				
		LWCommand *cmd2 = (LWCommand *) piece->getCommand();
		
		if (cmd2 != NULL) {
			if (TRUE == cmd2->hasReturn())
				cmd->setArgument (args++, NULL);
		} else {
			cmd->setArgument (args++, computeValue (n));
			destroy_list = g_slist_prepend (destroy_list, n);
		}
	}

	cmd->setArgc (args);	
	
	g_slist_foreach (destroy_list, (GFunc) g_node_destroy, NULL);
	g_slist_free (destroy_list);
}

/*! \brief Sprawdzenie poprawności polecenia parzystego
	
	Jeśli dwu członowe polecenie jest nie zamknięte zwraca wyjątek
	\param cmd Sprawdzane polecenie
*/
static void checkParity (LWParityCommand *cmd)
{
	if ((TRUE == cmd->isOpen() && FALSE == cmd->isAlreadyMatched())
		|| FALSE == cmd->isOpen())
			 throw new LWMessage (LW_ERROR_UnmatchedBrace);
}

/*! \brief Sprawdzanie poprawności stworzonego węzła
	
	Sprawdza poprawność poleceń, na które wskazuje węzeł.
	W przypadku wystąpień błędów, będą żucane wyjątki.
	Dodatkowo poleceniu zawartemu w węźle, przydzialane są 
	argumenty (te, które mogą być ustalone na etapie analizy
	programu, czyli nie będące rezulatatem wykonania innych poleceń).

	\param node Sprawdzany węzeł
*/
void LWProgram::checkNode (GNode *node)
{
	for (GNode *n = node; n != NULL; n = n->next) {
		LWPiece *piece = (LWPiece *) n->data;
		g_return_if_fail (piece != NULL);
	
		LWCommand *cmd = (LWCommand *) piece->getCommand();
		
		if (cmd == NULL)
			continue;
		
		if (TRUE == cmd->isParity())
			checkParity (dynamic_cast <LWParityCommand *> (cmd));
		
		try {
			setArguments (cmd, n->children);
		} catch (LWMessage *msg) {
			if (piece != begin_piece)
				msg->setPiece (piece);
			throw msg;			
		}
		
		checkNode (n->children);
	}
}

/* \brief Niszczy drzewo składniowe

	Niszczy drzewo składniowe oraz stos
*/
void LWProgram::uninitTree ()
{
	g_queue_free (context.stack);

	if (tree_piece != NULL) {
		g_node_traverse (tree_piece, G_POST_ORDER, G_TRAVERSE_ALL,
							-1, (GNodeTraverseFunc) unset_command, NULL);
	
		g_node_destroy (tree_piece);
	}
}

/* \brief Wyświetla komunikat

	Wyświetla komunikat
	\msg Komunikat do wyświetlenia
*/
void LWProgram::showMessage (LWMessage *msg)
{
		uninitTree ();
	
		msg->setProgram (this);
		msg->show();
		delete msg;
}

/*! \brief Analiza programu

	Przeprowadza analizę planszy z programem,
	tworząc drzewo składniowe. W razie błędów
	metoda tworzy okno z błędem i zaznacza ikonę
	w związku z którą wystąpił

	\param program Plansza z programem
	\return Prawda jęśli analiza zakończyła się sukcesem
	w przeciwnym wypadku fałsz.
*/
gboolean LWProgram::parse (LWBoard *program)
{
	g_return_val_if_fail (tree_piece == NULL, FALSE);
	
	program->unmarkPiece ();

	/* Pobranie listy klocków z poleceniami*/
	GSList *list_piece = program->getPieceList();

	/* Jeśli lista poleceń jest pusta, analizy nie trzeba przeprowadzać */
	if (list_piece == NULL)
		return TRUE;
	
	/* Wstawianie na początku programu klocka otwierającego
		blok logiczny */
	LWRow *row = new LWRow (program);
	begin_piece = new LWPiece (row);
	begin_piece->setCommand ("begin");
	GNode *node = g_node_new ((gpointer) begin_piece);

	LWPiece *prev = NULL;
	
	for (GSList *l = list_piece; l != NULL; l = l->next) {

		lex_scan (prev, (LWPiece *) l->data);
		prev = (LWPiece *) l->data;

		try {
			node = parse_iteration (node, (LWPiece *) l->data);
		} catch (LWMessage *msg) {
			msg->setPiece ( (LWPiece *) l->data);
			showMessage(msg);
			g_slist_free (list_piece);
			return FALSE;
		}
	
#if 0
		tree_piece = g_node_get_root (node); debugTree ();
#endif
		
	}
	
	LWPiece *end_piece = new LWPiece (row);
	end_piece->setCommand ("end");
	
	try {
		node = parse_iteration (node, end_piece);
	} catch (LWMessage *msg) {
			showMessage(msg);
			g_slist_free (list_piece);
			return FALSE;
	}

	delete end_piece;		
	
	g_slist_free (list_piece);
	
	tree_piece = g_node_get_root (node);
	
	try {
		checkNode (tree_piece);
	} catch (LWMessage *msg) {
		showMessage(msg);
		return FALSE;
	}
	
return TRUE;
}

/*! \brief Przydzielenie świata

	Przydziela świat, na którym będzie operował program.
	Metoda tworzy kopię tego świata i umieszcza w lewym górnym
	rogu czarodzieja, który po nim będzie się poruszał.

	\param a_world Plansza ze światem
*/
void LWProgram::setWorld (LWBoard *a_world)
{
	g_return_if_fail (context.wizard == NULL);
	g_return_if_fail (world == NULL);
	g_return_if_fail (a_world != NULL);
	g_return_if_fail (a_world->getType() == LW_TYPE_WORLD);
	g_return_if_fail (a_world->getRowNth(0) != NULL);
	g_return_if_fail (a_world->getRowNth(0)->getPieceNth(0) != NULL);
	
	world = new LWBoard (a_world, LW_TYPE_PLAYGROUND);
	
	GtkWidget *window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (window), _("Executing program..."));
	gtk_window_set_modal (GTK_WINDOW(window), TRUE);
	gtk_window_set_resizable (GTK_WINDOW(window), FALSE);
	
	if (NULL != LWEnvironment::getWidget()) {
		GtkWidget *parentwindow = gtk_widget_get_ancestor(
			LWEnvironment::getWidget(), GTK_TYPE_WINDOW);
		
		if (parentwindow != NULL)
			gtk_window_set_transient_for (GTK_WINDOW(window), GTK_WINDOW(parentwindow));
	}
	
    gtk_container_add (GTK_CONTAINER (window), world->getWidget());
	gtk_widget_show (window);

	g_signal_connect_swapped (G_OBJECT (window), "delete_event",
		G_CALLBACK (delete_cb), (gpointer) this);
	
	context.wizard = new LWWizard (world->getRowNth(0)->getPieceNth(0));
	context.wizard->restoreFromXML (LWEnvironment::getWizardXML());
}

/*! \brief Znajdź następne polecenie do wykonania
	
	Znajduje kolejne polecenie do wykonania w drzewie,
	na podstawie poprzednio wykonywanego polecenia,
	w większości przypadków będzie to node->next.
	Funkcja zmodyfikowana w ten sposób, że jeśli przedostatnim
	wykonywanym poleceniem była pętla, działanie będzie wracać
	do pętli.

	\param node Poprzednie polecenie w drzewie
	\return Kolejne polecenie do wykonania
*/
static GNode *get_next_node_to_execute (GNode *node)
{
	g_return_val_if_fail (node != NULL, NULL);
	
	if (node->prev != NULL) {
		LWPiece *piece = (LWPiece *) node->prev->data;
		g_return_val_if_fail (piece != NULL, NULL);
		
		LWCommand *pcmd = piece->getCommand();
		if (pcmd != NULL)
			if (pcmd->isLoop())
				return node->prev;
		}
	
	return node->next;
}

/*! \brief Pojednycza iteracja wykonywania poleceń w drzewie

	Oblicza polecenia, które dostarczają dane do polecenia -
	polecenia znajdujące się w węźle, a następnie wykonuje polecenie.
	Jest to pojedyncza iteracja przechodzenia drzewa w głąb (w
	kolejności Post Order )

	\param cmd Polecenie do wykonania
	\param children Polecenia do wykonania przed poleceniem cmd
*/
void LWProgram::executeOrderPost (LWCommand *cmd, GNode *children)
{
GNodePtr &ip = context.instrPtr;

	/* Zachowaj wartość ip */
	GNodePtr tmpip = ip;
	
	ip = children;
	
	while (ip != NULL) {
		GNodePtr lastip = ip;
		
		/* Wykonaj polecenia związane z argumentami */
		executeNode (ip);

		/* Znajdź następne polecenie do wykonania,
		   niektóre polecenia modyfikuja wskaźnik ip,
		   w takim przpadku ip jest już ustalone,
		   nie modyfikuj go */
		
		if (lastip == ip)
			ip = get_next_node_to_execute (ip);
	}

	/* Przywróć wartość ip */
	ip = tmpip;
		
	/* Wykonaj polecenie */
	cmd->execute (&context);
}

/*! \brief Wykonanie polecenie w węźle

	Wykonuje polecenia w węźle.

	\param node Węzeł do wykonania 
*/
void LWProgram::executeNode (GNode *node)
{
	LWPiece *piece = (LWPiece *) node->data;
	LWCommand *cmd = piece->getCommand();

	try {
		executeOrderPost (cmd, node->children);
	} catch (LWMessage *msg) {
		msg->setPiece(piece);
		throw msg;
	}
	
	while (gtk_events_pending ())
		gtk_main_iteration ();

	if (stoped == TRUE)
		throw new LWMessage (LW_INFO_ProgramInterrupted);
	
}

/*! \brief Wykonanie programu

	Wykonuje program. Metoda może być wykonana dopiero po stworzeniu
	drzewa składniowego przez metodę parse.

	\param node Węzeł do wykonania 
*/
void LWProgram::execute ()
{
	g_return_if_fail (world != NULL);

	GNodePtr &ip = context.instrPtr;
	ip = tree_piece;
	
	if (ip != NULL) {
		/* Wykonaj polecenie (blok logiczny) */
		try {
			executeNode (ip);
		} catch (LWMessage *msg) {
			showMessage(msg);
			finished = TRUE;
			return;
		}
	}
	finished = TRUE;
	uninitTree();
	
	GtkWidget *window = gtk_widget_get_ancestor(world->getWidget(), GTK_TYPE_WINDOW);
	gtk_window_set_title (GTK_WINDOW (window), _("Program finished..."));
	gtk_window_set_modal (GTK_WINDOW(window), FALSE);

}

/*! \brief Sprawdzenie czy program już się zakończył

	Sprawdza czy wykonywany program już się zakończył

	\return Rezultat testu 
*/
gboolean LWProgram::isFinished ()
{
	return finished;
}

/*! \brief Zatrzymanie programu

	Zatrzymuje program. Program może nie zostać zatrzymany od
	razu, zakończenie programu nastąpi po zakończeniu wykonywania
	bierzącego polecenia.
*/
void LWProgram::stop ()
{
	/* Przyśpiesz czarodzieja, żeby szybciej zakończył bierzącą instrukcję */
	if (context.wizard != NULL)
		context.wizard->setSpeed (9);
	
	stoped = TRUE;
}

/*! \brief Odpluskwanie drzewa

	Rysuje drzewo składniowe w nowym oknie. Metoda jest przydatna
	wyłącznie przy analizie poprawności algorytmów tworzących drzewo
	składniowe.
*/
void LWProgram::debugTree ()
{
	GtkWidget *w = widget_tree (tree_piece);

	GtkWidget *sw = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
				      GTK_POLICY_AUTOMATIC,
				      GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), w);
	
	GtkWidget *window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_add (GTK_CONTAINER (window), sw);
	gtk_widget_show_all (window);
}
