/* (C) Marcin Kwadrans <quar@vitea.pl> */

#include <stdlib.h>
#include "include/support.h"
#include "include/value.h"
#include "include/environment.h"

static int PixmapToInt (LWPixmap *pixmap)
{
	g_return_val_if_fail (pixmap != NULL, 0);
	
	gchar *s = pixmap->getName ();
	return atoi (s);
}

static LWPixmap *IntToPixmap (gint n)
{
	g_return_val_if_fail (NULL != LWEnvironment::getPixmapSet(), NULL);
	g_return_val_if_fail (n >= 0, NULL);
	
	gchar *s = g_strdup_printf ("%i", n);
	LWPixmap *pixmap = LWEnvironment::getPixmapSet()->getPixmap(s);
	g_free (s);
	return pixmap;
}

static LWPixmap *GetMinusSign ()
{
	g_return_val_if_fail (NULL != LWEnvironment::getPixmapSet(), NULL);
	return LWEnvironment::getPixmapSet()->getPixmap("-");
}

/*! \brief Konstruktor pustej wartości

	Tworzy pustą wartość
*/
LWValue::LWValue () : list_pixmap(NULL)
{
}

/*! \brief Konstruktor wartości z ikoną

	Tworzy wartość zawierającą pojedynczą ikonę
*/
LWValue::LWValue (LWPixmap *pixmap)
{
	list_pixmap = g_slist_append (NULL, pixmap);
}

/*! \brief Konstruktor wartości na podstawie liczy

	Tworzy wartość zawierającą listę ikon będącą odpowiednikiem liczby
	\param ivalue 
*/
LWValue::LWValue (gint ivalue) : list_pixmap(NULL)
{
	gboolean hasminus = (ivalue < 0) ? TRUE : FALSE;
	
	if (hasminus == TRUE)
		ivalue = - ivalue;
	
	do {
		LWPixmap *pixmap = IntToPixmap(ivalue % 10);
		list_pixmap = g_slist_prepend (list_pixmap, (gpointer) pixmap);
		 ivalue /= 10;
	} while (ivalue > 0);
	
	if (hasminus == TRUE) {
		LWPixmap *pixmap = GetMinusSign ();
		list_pixmap = g_slist_prepend (list_pixmap, (gpointer) pixmap);
	}
}

/*! \brief Destruktor

	Niszczy wartość
*/	
LWValue::~LWValue ()
{
	clear();
}

/*! \brief Czyszczenie wartości

	Czyści wartość niszcząc przy tym wszystkie ikony wchodzące w jej skład
*/	
void LWValue::clear ()
{
	g_slist_free (list_pixmap);
	list_pixmap = NULL; 
}

/*! \brief Dołączenie ikony

	Dołącza ikonę do końca listy przechywowanej w wartości
	\param pixmap Ikona
*/
void LWValue::append (LWPixmap *pixmap)
{
	list_pixmap = g_slist_append (list_pixmap, pixmap);
}

/*! \brief Łączenie wartości

	Dołącza listę przechywaną w value na koniec listy przechywowanej w wartości
	\param value Warość dołączana
*/
void LWValue::concat (LWValue *value)
{
	GSList *pixmaps = g_slist_copy (value->list_pixmap);
	list_pixmap = g_slist_concat (pixmaps, list_pixmap);
}

/*! \brief Interpretacja jako liczba

	Interpretuje wartość jako liczbę
	\return Liczba
*/
int LWValue::toInt ()
{
gboolean isminus = FALSE;
gint ivalue = 0;
GSList *l;
	
	for (l = list_pixmap; l != NULL; l = l->next) {
		if (GetMinusSign () == l->data) isminus = TRUE;
		if (PixmapToInt( (LWPixmap *) l->data) > 0) break;
		}
		
	for (; l != NULL; l = l->next) {
		ivalue *= 10; 
		ivalue += PixmapToInt( (LWPixmap *) l->data);
	}
			
	return (isminus == FALSE) ? ivalue : -ivalue;
}

/*! \brief Pobiera listę ikon

	Pobiera listę ikon znajdującą się w wartości
	\return Pobrana lista
*/
GSList *LWValue::getListPixmap ()
{
	return g_slist_copy (list_pixmap);	
}

/*! \brief Ustawia wartość

	Ustawia wartość na podstawie innej wartości
	\param value Wartość na podstawie której ma być dokonane podstawienie
*/
void LWValue::set (LWValue *value)
{
	if (this == value) return;
		
	clear ();
	
	list_pixmap = g_slist_copy (value->list_pixmap);
}

/*! \brief Czy wartość stanowi zmienną

	Czy wartość stanowi zmienną
	\return Nie przeciążona metoda zawsze zwraca fałsz
*/
gboolean LWValue::isVariable ()
{
	return FALSE;
}
